effectful-core
Safe HaskellSafe-Inferred
LanguageGHC2021

Effectful.Output.Static.Action

Description

Support for feeding values of a particular type to a monadic action.

Since: 2.7.0.0

Synopsis

Effect

data Output (o :: Type) :: Effect Source #

Provide the ability to feed values of type o to a monadic action.

Instances

Instances details
type DispatchOf (Output o) Source # 
Instance details

Defined in Effectful.Output.Static.Action

data StaticRep (Output o) Source # 
Instance details

Defined in Effectful.Output.Static.Action

data StaticRep (Output o) where

Handlers

runOutput Source #

Arguments

:: forall o es a. HasCallStack 
=> (HasCallStack => o -> Eff es ())

The action for receiving values.

-> Eff (Output o : es) a 
-> Eff es a 

Run the Output effect with the given action for receiving values.

Operations

output Source #

Arguments

:: (HasCallStack, Output o :> es) 
=> o

The value.

-> Eff es () 

Feed the value to the underlying monadic action.