effectful-core
Safe HaskellSafe-Inferred
LanguageGHC2021

Effectful.Output.Static.Local.List

Description

Support for accumulation of values in a thread local list.

Since: 2.7.0.0

Synopsis

Effect

data Output (o :: Type) :: Effect Source #

Provide access to accumulation of values of type o in a thread local list.

Instances

Instances details
type DispatchOf (Output o) Source # 
Instance details

Defined in Effectful.Output.Static.Local.List

newtype StaticRep (Output o) Source # 
Instance details

Defined in Effectful.Output.Static.Local.List

newtype StaticRep (Output o) = Output [o]

Handlers

runOutput :: HasCallStack => Eff (Output o : es) a -> Eff es (a, [o]) Source #

Run the Output effect and return the final value along with the accumulated list.

Operations

output Source #

Arguments

:: (HasCallStack, Output o :> es) 
=> o

The value.

-> Eff es () 

Append the value to the end of the list.