| Safe Haskell | Safe-Inferred |
|---|---|
| Language | GHC2021 |
Effectful.Input.Static.Action
Contents
Description
Support for access to values supplied by a monadic action.
Since: 2.7.0.0
Synopsis
- data Input (i :: Type) :: Effect
- runInput :: forall i es a. HasCallStack => (HasCallStack => Eff es i) -> Eff (Input i : es) a -> Eff es a
- input :: (HasCallStack, Input i :> es) => Eff es i
- inputs :: (HasCallStack, Input i :> es) => (i -> a) -> Eff es a
Effect
data Input (i :: Type) :: Effect Source #
Provide access to values of type i supplied by a monadic action.
Instances
| type DispatchOf (Input i) Source # | |
Defined in Effectful.Input.Static.Action | |
| data StaticRep (Input i) Source # | |
Handlers
Arguments
| :: forall i es a. HasCallStack | |
| => (HasCallStack => Eff es i) | The action for input generation. |
| -> Eff (Input i : es) a | |
| -> Eff es a |
Run the Input effect with the given action that supplies values.