| Safe Haskell | Safe-Inferred |
|---|---|
| Language | GHC2021 |
Effectful.Output.Static.Action
Contents
Description
Support for feeding values of a particular type to a monadic action.
Since: 2.7.0.0
Synopsis
- data Output (o :: Type) :: Effect
- runOutput :: forall o es a. HasCallStack => (HasCallStack => o -> Eff es ()) -> Eff (Output o : es) a -> Eff es a
- output :: (HasCallStack, Output o :> es) => o -> Eff es ()
Effect
data Output (o :: Type) :: Effect Source #
Provide the ability to feed values of type o to a monadic action.
Instances
| type DispatchOf (Output o) Source # | |
Defined in Effectful.Output.Static.Action | |
| data StaticRep (Output o) Source # | |
Handlers
Arguments
| :: forall o es a. HasCallStack | |
| => (HasCallStack => o -> Eff es ()) | The action for receiving values. |
| -> Eff (Output o : es) a | |
| -> Eff es a |
Run the Output effect with the given action for receiving values.