| Safe Haskell | Safe-Inferred |
|---|---|
| Language | GHC2021 |
Effectful.Output.Static.Local.List
Contents
Description
Support for accumulation of values in a thread local list.
Since: 2.7.0.0
Effect
data Output (o :: Type) :: Effect Source #
Provide access to accumulation of values of type o in a thread local
list.
Instances
| type DispatchOf (Output o) Source # | |
Defined in Effectful.Output.Static.Local.List | |
| newtype StaticRep (Output o) Source # | |
Defined in Effectful.Output.Static.Local.List | |
Handlers
runOutput :: HasCallStack => Eff (Output o : es) a -> Eff es (a, [o]) Source #
Run the Output effect and return the final value along with the
accumulated list.